use hd44780_driver::HD44780;
use embedded_hal::digital::v2::OutputPin;
use core::fmt::Write;

use crate::hw::Delay;
use crate::logic::State;
use crate::view;

pub struct Display<B: hd44780_driver::bus::DataBus> {
    lcd: HD44780<B>,
    delay: Delay,
    layout: Layout,
}

impl <RS, EN, D4, D5, D6, D7> Display<hd44780_driver::bus::FourBitBus<RS, EN, D4, D5, D6, D7>>
where
    RS: OutputPin, EN: OutputPin, D4: OutputPin,
    D5: OutputPin, D6: OutputPin, D7: OutputPin,
{
    pub fn new(rs: RS, en: EN, d4: D4, d5: D5, d6: D6, d7: D7) -> Self
    {
        let mut delay = Delay::new();
        let mut lcd = HD44780::new_4bit(rs, en, d4, d5, d6, d7, &mut delay).unwrap();
        lcd.reset(&mut delay).ok();
        lcd.clear(&mut delay).ok();
        lcd.set_display_mode(hd44780_driver::display_mode::DisplayMode {
            cursor_visibility: hd44780_driver::Cursor::Invisible,
            cursor_blink: hd44780_driver::CursorBlink::Off,
            display: hd44780_driver::Display::On,
        }, &mut delay).ok();
        Display {
            lcd,
            delay,
            layout: Layout::Empty,
        }
    }
}

impl <B: hd44780_driver::bus::DataBus> Display<B> {
    fn pos(&self, x: u8, y: u8) -> u8 {
        match y {
            0 => x,
            1 => x + 64,
            2 => x + 20,
            3 => x + 84,
            _ => 0,
        }
    }
    fn write_str_at(&mut self, x: usize, y: usize, s: &str) {
        if x > 19 || y > 3 {
            return
        }
        let mut len = 20 - x;
        if len as usize > s.len() {
            len = s.len();
        }

        let x = x as u8;
        let y = y as u8;
        self.lcd.set_cursor_pos(self.pos(x, y), &mut self.delay).ok();
        let len = len as usize;
        self.lcd.write_str(&s[..len], &mut self.delay).ok();
    }
    pub fn write(&mut self, l: Layout) {
        match l {
            Layout::Empty => {
                if let Layout::Empty = self.layout {
                    return;
                }
                self.lcd.clear(&mut self.delay).ok();
            },
            Layout::Splash => {
                self.lcd.clear(&mut self.delay).ok();
                self.write_str_at((20 - SPLASH_1.len()) / 2, 1, SPLASH_1);
                self.write_str_at((20 - SPLASH_2.len()) / 2, 2, SPLASH_2);
            },
            Layout::Status(ref cur) => {
                let mut temperature_changed = true;
                let mut time_left_changed = true;
                let mut state_changed = true;
                let mut speed_changed = true;
                if let Layout::Status(ref prev) = &self.layout {
                    temperature_changed = prev.temperature != cur.temperature;
                    time_left_changed = prev.time_left != cur.time_left;
                    state_changed = prev.state != cur.state;
                    speed_changed = prev.speed != cur.speed;
                } else {
                    self.lcd.clear(&mut self.delay).ok();
                    self.write_str_at(0, 0, "      State");
                    self.write_str_at(0, 1, "Temperature");
                    self.write_str_at(0, 2, "      Speed");
                    self.write_str_at(0, 3, "  Time Left");
                }

                let mut s: heapless::String<16> = heapless::String::new();

                if state_changed {
                    let s = match cur.state {
                        State::Idle => "Idle   ",
                        State::Preheating => "Heating",
                        State::Running => "Running",
                    };
                    self.write_str_at(12, 0, s);
                }
                if temperature_changed {
                    s.clear();
                    write!(s, "{}.{}C", cur.temperature / 10, cur.temperature % 10).ok();
                    while s.len() < 8 {
                        s.push(' ').ok();
                    }
                    self.write_str_at(12, 1, &s);
                }
                if speed_changed {
                    s.clear();
                    write!(s, "{}mm/s", cur.speed).ok();
                    while s.len() < 8 {
                        s.push(' ').ok();
                    }
                    self.write_str_at(12, 2, &s);
                }
                if time_left_changed {
                    s.clear();
                    let h = cur.time_left / 3600;
                    let m = (cur.time_left / 60) % 60;
                    let ss = cur.time_left % 60;
                    write!(s, "{:02}:{:02}:{:02}", h, m, ss).ok();
                    while s.len() < 8 {
                        s.push(' ').ok();
                    }
                    self.write_str_at(12, 3, &s);
                }
            },
            Layout::Menu(ref cur) => {
                let redraw_options = match &self.layout {
                    Layout::Menu(LayoutMenu { name, .. }) => *name != cur.name,
                    _ => true,
                };
                let redraw_cursor = match &self.layout {
                    Layout::Menu(LayoutMenu { pos, .. }) => *pos != cur.pos,
                    _ => true,
                };

                if redraw_options {
                    self.lcd.clear(&mut self.delay).ok();
                    for (i, option) in cur.options.iter().enumerate() {
                        self.write_str_at(1, i, option.label);
                    }
                }
                if redraw_cursor {
                    for (i, _) in cur.options.iter().enumerate() {
                        if i == cur.pos {
                            self.write_str_at(0, i, ">");
                        } else {
                            self.write_str_at(0, i, " ");
                        }
                    }
                }
            },
        }
        self.layout = l;
    }
}

pub struct LayoutStatus {
    pub temperature: u16,
    pub time_left: u32,
    pub state: State,
    pub speed: u16,
}

pub struct LayoutMenu {
    pub name: view::ViewID,
    pub pos: usize,
    pub options: &'static [view::MenuOption],
}

pub enum Layout {
    Empty,
    Splash,
    Status(LayoutStatus),
    Menu(LayoutMenu),
}

const SPLASH_1: &'static str = "q3k's pullstruder";
const SPLASH_2: &'static str = "2023/10/07";

