#[derive(PartialEq, Eq, Clone, Copy)]
pub enum State {
    Idle,
    Preheating,
    Running,
}

#[derive(Clone)]
pub enum UserRequest {
    Start,
    Adjust {
        speed: Option<u16>,
        temperature: Option<u16>,
        time: Option<u32>,
    },
    Stop,
}

pub struct Input {
    pub timestamp: u32,
    pub temperature: u16,
    pub request: Option<UserRequest>,
}

pub struct Controller {
    pub state: State,
    deadline: u32,
    configured_seconds: u32,
    last_timestamp: u32,
    pub speed: u16,
    pub temperature: u16,
}

impl Controller {
    pub fn new() -> Self {
        Self {
            state: State::Idle,
            deadline: 0,
            configured_seconds: 3600,
            last_timestamp: 0,
            speed: 300,
            temperature: 2137,
        }
    }

    pub fn time_left(&self) -> u32 {
        match self.state {
            State::Running => {
                if self.last_timestamp > self.deadline {
                    return 0;
                }
                self.deadline - self.last_timestamp
            },
            _ => self.configured_seconds,
        }
    }

    pub fn process(&mut self, i: &Input) {
        self.last_timestamp = i.timestamp;
        match self.state {
            State::Idle => self.process_idle(i),
            State::Preheating => self.process_preheating(i),
            State::Running => self.process_running(i),
        }
    }

    fn apply_adjust(&mut self, speed: Option<u16>, temperature: Option<u16>, time: Option<u32>) {
        if let Some(speed) = speed {
            self.speed = speed;
        }
        if let Some(temperature) = temperature {
            self.temperature = temperature;
        }
        if let Some(time) = time {
            self.configured_seconds = time;
        }
    }

    fn process_idle(&mut self, i: &Input) {
        if let Some(req) = &i.request {
            match req {
                UserRequest::Start => {
                    self.deadline = i.timestamp + self.configured_seconds;
                    self.state = State::Preheating;
                },
                UserRequest::Adjust { speed, temperature, time } => {
                    self.apply_adjust(speed.clone(), temperature.clone(), time.clone());
                },
                UserRequest::Stop => {},
            }
        }
    }

    fn process_preheating(&mut self, i: &Input) {
        if let Some(req) = &i.request {
            match req {
                UserRequest::Start {..} => {},
                UserRequest::Adjust { speed, temperature, time } => {
                    self.apply_adjust(speed.clone(), temperature.clone(), time.clone());
                },
                UserRequest::Stop => {
                    self.state = State::Idle;
                },
            }
        }
    }

    fn process_running(&mut self, i: &Input) {
        if let Some(req) = &i.request {
            match req {
                UserRequest::Start {..} => {},
                UserRequest::Adjust { speed, temperature, time } => {
                    self.apply_adjust(speed.clone(), temperature.clone(), time.clone());
                },
                UserRequest::Stop => {
                    self.state = State::Idle;
                },
            }
        }
    }
}

