#![no_std]
#![no_main]
#![feature(abi_avr_interrupt)]

use panic_halt as _;

mod key;
mod view;
mod display;
mod logic;
mod hw;

/// Seconds elapsed since startup. Overflows after 136 years of uptime.
static mut TICKS_SEC: u32 = 0;

#[avr_device::interrupt(atmega1284p)]
fn TIMER1_COMPA() {
    unsafe {
        TICKS_SEC += 1;
        if TICKS_SEC == 0 {
            panic!("have you been running this non-stop for 136 years????");
        }
    }
}

fn ticks_sec() -> u32 {
    let v = unsafe {
        TICKS_SEC
    };
    return v;
}

#[avr_device::entry]
fn main() -> ! {
    let dp = atmega_hal::Peripherals::take().unwrap();
    let pins = atmega_hal::pins!(dp);


    let mut led = pins.pa4.into_output().downgrade();

    let mut display = display::Display::new(
        pins.pa3.into_output(), // RS
        pins.pa2.into_output(), // EN
        pins.pd2.into_output(), // D4
        pins.pd3.into_output(), // D5
        pins.pc0.into_output(), // D6
        pins.pc1.into_output(), // D7
    );

    let mut adc = atmega_hal::Adc::<hw::Clock>::new(dp.ADC, Default::default());
    let menu_pin = pins.pa1.into_analog_input(&mut adc);

    // Configure TMR1 at 1Hz.
    let tmr1 = dp.TC1;
    tmr1.tccr1a.write(|w| w.wgm1().bits(0b00));
    tmr1.tccr1b.write(|w| w.cs1().prescale_1024().wgm1().bits(0b01));
    tmr1.ocr1a.write(|w| w.bits(15624));
    tmr1.timsk1.write(|w| w.ocie1a().set_bit());

    unsafe {
        avr_device::interrupt::enable();
    }

    display.write(display::Layout::Splash);
    hw::delay_ms(1000);

    let mut ctrl = logic::Controller::new();
    let mut menu = view::ViewManager::new();
    let mut prev_key: Option<key::Key> = None;

    loop {
        let ticks = ticks_sec();

        // UI
        let key = key::parse(adc.read_blocking(&menu_pin));
        let mut menu_input = view::ViewInput {
            key: None,
        };
        if key != prev_key {
            menu_input.key = key;
        };
        prev_key = key;
        let req = menu.process(&menu_input);
        display.write(menu.layout(&ctrl));

        // Logic
        let input = logic::Input {
            timestamp: ticks,
            temperature: 2137,
            request: req,
        };
        ctrl.process(&input);
        led.toggle();

    }
}
