use crate::key::Key;
use crate::logic::{Controller, UserRequest};
use crate::display::{Layout, LayoutStatus, LayoutMenu};

pub struct ViewInput {
    pub key: Option<Key>,
}

#[derive(PartialEq, Eq, Clone, Copy)]
pub enum ViewID {
    Status,
    Main,
}

#[derive(Clone)]
pub struct MenuAction {
    goto_view: Option<ViewID>,
    logic_request: Option<UserRequest>,
}

impl MenuAction {
    fn nothing() -> Self {
        Self {
            goto_view: None,
            logic_request: None,
        }
    }

    fn just_goto_view(vid: ViewID) -> Self {
        Self {
            goto_view: Some(vid),
            logic_request: None,
        }
    }

    fn just_logic_request(req: UserRequest) -> Self {
        Self {
            goto_view: None,
            logic_request: Some(req),
        }
    }
}

pub trait View {
    fn layout(&self, controller: &Controller) -> Layout;
    fn process(&mut self, input: &ViewInput) -> MenuAction;
    fn enter(&mut self) { }
}

pub struct StatusView;

impl View for StatusView {
    fn layout(&self, controller: &Controller) -> Layout {
        Layout::Status(LayoutStatus {
            temperature: controller.temperature,
            time_left: controller.time_left(),
            state: controller.state,
            speed: controller.speed,
        })
    }
    fn process(&mut self, input: &ViewInput) -> MenuAction {
        if input.key.is_some() {
            return MenuAction::just_goto_view(ViewID::Main);
        }
        return MenuAction::nothing();
    }
}

struct Menu {
    id: ViewID,
    pos: usize,
    options: &'static [MenuOption],
}

pub struct MenuOption {
    pub label: &'static str,
    action: MenuAction,
}

impl View for Menu {
    fn layout(&self, _controller: &Controller) -> Layout {
        Layout::Menu(LayoutMenu {
            name: self.id,
            pos: self.pos,
            options: self.options,
        })
    }

    fn process(&mut self, input: &ViewInput) -> MenuAction{
        match input.key {
            Some(Key::Up) => { if self.pos > 0 { self.pos -= 1; } },
            Some(Key::Down) => { if self.pos < self.options.len()-1 { self.pos += 1; } },
            Some(Key::Select) => { return self.options[self.pos].action.clone() },
            _ => (),
        }
        return MenuAction::nothing();
    }
    fn enter(&mut self) {
        self.pos = 0;
    }
}

pub struct ViewManager {
    status: StatusView,
    main: Menu,
    cur: ViewID,
}

impl ViewManager {
    pub fn new() -> Self {
        Self {
            status: StatusView,
            main: Menu {
                id: ViewID::Main,
                pos: 0,
                options: &[
                    MenuOption {
                        label: "Back",
                        action: MenuAction {
                            goto_view: Some(ViewID::Status),
                            logic_request: None,
                        },
                    },
                    MenuOption {
                        label: "Start",
                        action: MenuAction {
                            goto_view: Some(ViewID::Status),
                            logic_request: Some(UserRequest::Start),
                        },
                    },
                ],
            },
            cur: ViewID::Status,
        }
    }

    pub fn layout(&self, controller: &Controller) -> Layout {
        match self.cur {
            ViewID::Status => self.status.layout(controller),
            ViewID::Main => self.main.layout(controller),
        }
    }

    pub fn process(&mut self, input: &ViewInput) -> Option<UserRequest> {
        let new = match self.cur {
            ViewID::Status => self.status.process(input),
            ViewID::Main => self.main.process(input),
        };
        if let Some(id) = new.goto_view {
            self.cur = id;
            match self.cur {
                ViewID::Status => self.status.enter(),
                ViewID::Main => self.main.enter(),
            }
        }
        return new.logic_request;
    }
}

